@@ -3,6 +3,7 @@ import logging
 import re
 from typing import (
     TYPE_CHECKING,
+    Any,
     Dict,
     List,
     Optional,
@@ -170,7 +171,7 @@ class SearchContext(object):
         for sp in search_params_def:
             # Look out for any composite or combo type parameter
             if sp.type == "composite":
-                normalized_params.append(
+                normalized_params.extend(
                     self._normalize_composite_param(
                         raw_value, param_def=sp, modifier=modifier_
                     )
@@ -182,8 +183,7 @@ class SearchContext(object):
             elif len(raw_value) == 1:
                 raw_value = raw_value[0]
 
-            values: List = list()
-            self.normalize_param_value(raw_value, sp, values)
+            values: List = self.normalize_param_value(raw_value, sp)
 
             if len(values) == 0:
                 # empty parameters are not considered an error, they should be ignored
@@ -199,20 +199,20 @@ class SearchContext(object):
         return normalized_params
 
     def normalize_param_value(
-        self, raw_value: Union[List, str], search_param: SearchParameter, container
+        self, raw_value: Union[List, str], search_param: SearchParameter
     ):
-        """ """
+        normalized_values: List[Any] = []
         if not raw_value:
-            return
+            return []
 
         elif isinstance(raw_value, list):
             bucket: List[str] = list()
             for rv in raw_value:
-                self.normalize_param_value(rv, search_param, bucket)
+                bucket.extend(self.normalize_param_value(rv, search_param))
             if len(bucket) == 1:
-                container.append(bucket[0])
+                normalized_values.append(bucket[0])
             else:
-                container.append(bucket)
+                normalized_values.append(bucket)
 
         else:
             escape_ = has_escape_comma(raw_value)
@@ -237,9 +237,11 @@ class SearchContext(object):
                         break
                 bucket_.append((comparison_operator, val_))
             if len(bucket_) == 1:
-                container.append(bucket_[0])
+                normalized_values.append(bucket_[0])
             else:
-                container.append((None, bucket_))
+                normalized_values.append((None, bucket_))
+
+        return normalized_values
 
     def _get_search_param_definitions(self, param_name) -> List[SearchParameter]:
         """ """
@@ -265,51 +267,51 @@ class SearchContext(object):
         path_.finalize(self.engine)
         return path_
 
-    def _normalize_composite_param(self, raw_value, param_def, modifier):
+    def _normalize_composite_param(
+        self, raw_value, param_def, modifier
+    ) -> List[Tuple[ElementPath, str, Optional[str]]]:
         """ """
         if len(raw_value) < 1:
             raise NotImplementedError(
                 "Currently duplicate composite type params are not allowed or supported"
             )
-        value_parts = raw_value[0].split("&")
-        assert len(value_parts) == 2
-
-        composite_bucket = list()
-
-        part1 = [
-            ".".join([param_def.expression, param_def.component[0]["expression"]]),
-            value_parts[0],
-        ]
-        part1_param_value = list()
-        self.normalize_param_value(part1[1], param_def, part1_param_value)
-        if len(part1_param_value) == 1:
-            part1_param_value = part1_param_value[0]
-        composite_bucket.append(
-            (self._dotted_path_to_path_context(part1[0]), part1_param_value, modifier)
-        )
-        part2 = list()
-        for expr in param_def.component[1]["expression"].split("|"):
-            part_ = [
-                ".".join([param_def.expression, expr.strip()]),
-                value_parts[1],
-            ]
-            part2.append(part_)
-        part2_param_value = list()
-        self.normalize_param_value(part2[0][1], param_def, part2_param_value)
-
-        if len(part2_param_value) == 1:
-            part2_param_value = part2_param_value[0]
-        part2_temp = list()
-        for pr in part2:
-            part2_temp.append(
-                (self._dotted_path_to_path_context(pr[0]), part2_param_value, modifier)
+        value_parts = raw_value[0].split("$")
+        if len(value_parts) != len(param_def.component):
+            raise ValueError(
+                f"Composite search param {param_def.name} expects {len(param_def.component)} "
+                f"values separated by a '$', got {len(value_parts)}."
             )
-        if len(part2_temp) == 1:
-            part2_temp = part2_temp[0]
 
-        composite_bucket.append(part2_temp)
+        results: List[Tuple[ElementPath, str, Optional[str]]] = [
+            self.parse_composite_parameter_component(
+                component, value_part, param_def, modifier
+            )
+            for component, value_part in zip(param_def.component, value_parts)
+        ]
+        return results
 
-        return composite_bucket
+    def parse_composite_parameter_component(
+        self, component, raw_value, param_def, modifier
+    ):
+        result = []
+        for expr in component["expression"].split("|"):
+            component_dotted_path = ".".join([param_def.expression, expr.strip()])
+
+            component_param_value = self.normalize_param_value(raw_value, param_def)
+            if len(component_param_value) == 1:
+                component_param_value = component_param_value[0]
+
+            result.append(
+                (
+                    self._dotted_path_to_path_context(component_dotted_path),
+                    component_param_value,
+                    modifier,
+                )
+            )
+
+        if len(result) == 1:
+            return result[0]
+        return result
 
 
 @implementer(ISearch)
@@ -712,7 +714,13 @@ class Search(object):
                 terms = list()
                 for nd in normalized_data:
                     self.add_term(nd, terms)
-                return G_(*terms, path=None, type_=GroupType.DECOUPLED)
+                # I think we'll be there only in the case of composite search params
+                # The Group path is only needed to build nested queries so using
+                # whichever component path should be ok.
+                # This could still use a refacto though...
+                group_term = G_(*terms, path=nd[0], type_=GroupType.COUPLED)
+                terms_container.append(group_term)
+                return group_term
 
             else:
                 normalized_data = normalized_data[0]
@@ -767,8 +775,10 @@ class Search(object):
             term = term_factory(path_, param_value, modifier)
         else:
             term = self.create_term(path_, param_value, modifier)
-
-        terms_container.append(term)
+        if isinstance(term, list):
+            terms_container.extend(term)
+        else:
+            terms_container.append(term)
 
     def create_identifier_term(self, path_, param_value, modifier):
         """ """
@@ -777,7 +787,7 @@ class Search(object):
                 self.create_identifier_term(path_, value, modifier)
                 for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_identifier_term(path_, param_value, modifier)
@@ -850,7 +860,7 @@ class Search(object):
                 self.create_quantity_term(path_, value, modifier)
                 for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_quantity_term(path_, param_value, modifier)
@@ -929,7 +939,7 @@ class Search(object):
             terms = [
                 self.create_coding_term(path_, value, modifier) for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_coding_term(path_, param_value, modifier)
@@ -1005,7 +1015,7 @@ class Search(object):
                 self.create_codeableconcept_term(path_, value, modifier)
                 for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_codeableconcept_term(path_, param_value, modifier)
@@ -1051,7 +1061,7 @@ class Search(object):
             terms = [
                 self.create_address_term(path_, val, modifier) for val in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_address_term(path_, param_value, modifier)
@@ -1093,7 +1103,7 @@ class Search(object):
                 self.create_contactpoint_term(path_, val, modifier)
                 for val in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_contactpoint_term(path_, param_value, modifier)
@@ -1144,7 +1154,7 @@ class Search(object):
             terms = [
                 self.create_humanname_term(path_, val, modifier) for val in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)  # Term or Group
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_humanname_term(path_, param_value, modifier)
@@ -1187,7 +1197,7 @@ class Search(object):
                 self.create_reference_term(path_, value, modifier)
                 for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_reference_term(path_, param_value, modifier)
@@ -1243,7 +1253,7 @@ class Search(object):
             terms = [
                 self.create_money_term(path_, value, modifier) for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_money_term(path_, param_value, modifier)
@@ -1314,7 +1324,7 @@ class Search(object):
                 self.single_valued_period_term(path_, value, modifier)
                 for value in param_value
             ]
-            return G_(*terms, path=path_, type_=GroupType.COUPLED)
+            return terms
 
         elif isinstance(param_value, tuple):
             return self.single_valued_period_term(path_, param_value, modifier)
@@ -1579,7 +1589,7 @@ class Search(object):
             return builder
 
         specs = [
-            lookup_fhir_resource_spec(r, True, FHIR_VERSION.R4)
+            lookup_fhir_resource_spec(r, True, self.context.engine.fhir_release)
             for r in self.context.resource_types
         ]
 
